#!/usr/bin/env python

"""
Some useful functions for number theory
"""

__all__ = ['divisors', 'is_prime', 'primes', 'pi', 'first_primes', 'prime_divisors']

def divisors(N):
    """divisors(N) -> tuple of divisors of the integer N
	Example:
	>>> divisors(2009)
	(7, 41, 49, 287, 2009)
	>>> divisors(2011)
	(2011)
	>>>
"""
    l = list()
    for i in range(2, N+1):
        if N%i == 0:
            # i is a divisor!
            l.append(i)
    return tuple(l)

def is_prime(N):
    """is_prime(N) -> primality test of N
	Example:
	>>> is_prime(7)
	True
	>>> is_prime(9)
	False
	>>>
"""
    for j in range(2, N):
        if N%j == 0:
            # i is not prime!
            return False
    else:
        return True

def prime_divisors(N):
    """prime_divisors(N) -> tuple of prime divisors of the integer N
	Example:
	>>> prime_divisors(2009)
	(7, 41)
	>>> prime_divisors(2011)
	(2011)
	>>>
"""
    l = list()
    for d in divisors(N):
        if is_prime(d):
            l.append(d)
    return tuple(l)

def primes():
    """primes() -> generate all prime numbers
	Example:
>>> for i in primes():
...     if i > 10:
...          break
...     print i
...
2 3 5 7
>>>
"""
    yield 2
    i = 3
    while True:
        if is_prime(i):
            yield i
        i += 2

def pi(N):
    """pi(N) -> number of primes less than N
	Example:
>>> print pi(10)
4
>>>
"""
    count = 0
    for p in primes():
        if p >= N:
            break
        count += 1
    return count

def repeat(n, v=None):
    i = 0
    while i < n:
        yield v
        i += 1

def first_primes(N):
    """first_primes(N) -> iterate over the first N primes
	Example:
	>>> for i in first_primes(4):
	...     print i
        ...
        2
	3
	5
	7
	>>>
"""
    for dummy, p in zip(repeat(N), primes()):
        yield p


if __name__ == "__main__":
    print "===", 2009, prime_divisors(2009)
    print "===", 2011, prime_divisors(2011)

    for i in primes():
        print i
        if i > 1000:
             break

    def repeat(n, v=None):
        i = 0
        while i < n:
            yield v
            i += 1
        
    print [t[1] for t in zip(repeat(250), primes())]

#    l = []
#    p = primes()
#    while len(l) < 250:
#        l.append(next(p))
#    print l
